#include "drawing.h"

u16 gfx[20];

void initGraphics(void)
{
    PA_Init3D();
    PA_Reset3DSprites();

    gfx[Black] = PA_3DCreateTex((void*)BlackStone_Texture, 16, 16, TEX_256COL);
    gfx[White] = PA_3DCreateTex((void*)WhiteStone_Texture, 16, 16, TEX_256COL);
    gfx[Black_T] = PA_3DCreateTex((void*)Black_T_Texture, 16, 16, TEX_256COL);
    gfx[White_T] = PA_3DCreateTex((void*)White_T_Texture, 16, 16, TEX_256COL);
    gfx[Quit] = PA_3DCreateTex((void*)quit_Texture,64,32, TEX_256COL);
    gfx[Save] = PA_3DCreateTex((void*)save_Texture,64,32, TEX_256COL);
    gfx[List] = PA_3DCreateTex((void*)list_Texture,64,32, TEX_256COL);
    gfx[Score] = PA_3DCreateTex((void*)score_Texture,64,32, TEX_256COL);
    gfx[Up] = PA_3DCreateTex((void*)up_Texture,16,16, TEX_256COL);
    gfx[Down] = PA_3DCreateTex((void*)down_Texture,16,16, TEX_256COL);
    gfx[Left] = PA_3DCreateTex((void*)left_Texture,16,16, TEX_256COL);
    gfx[Right] = PA_3DCreateTex((void*)right_Texture,16,16, TEX_256COL);
    gfx[Wrong] = PA_3DCreateTex((void*)Wrong_Texture,64,64, TEX_256COL);
    gfx[Ok] = PA_3DCreateTex((void*)OK_Texture,64,64, TEX_256COL);

    PA_Load3DSpritePal(0, (void*)Sprite3D_Pal);
}

void drawStone(int x, int y, int color)
{
    PA_3DCreateSpriteFromTex(20+x+19*y, 					// Sprite number
                             gfx[color==BLACK?Black:White], 				// Gfx...
                             10, 10, 				// Width, Height
                             0,   		// Palette, unused in 16bit...
                             8+10*x, 8+10*y); // X, Y SPRITE CENTER !
}
void drawCountPoint(int x, int y, int color)
{
    PA_3DCreateSpriteFromTex(20+x+19*y, 					// Sprite number
                             gfx[color==BLACK?Black_T:White_T], 				// Gfx...
                             10, 10, 				// Width, Height
                             0,   		// Palette, unused in 16bit...
                             8+10*x, 8+10*y); // X, Y SPRITE CENTER !
}

void eraseStone(int x, int y)
{
    PA_3DDeleteSprite(20+x+19*y);
}

void eraseAllStone(void)
{
    int i,j;
    for (i=0; i<19;i++)
        for(j=0;j<19;j++)
            eraseStone(i,j);
}

void eraseAll(void)
{
    int i;
    for (i=0; i<400; i++)
        PA_3DDeleteSprite(i);
    PA_3DProcess ();
}

void drawRecordGameInterface(void)
{
    PA_3DCreateSpriteFromTex(Left,gfx[Left],16,16,0,218,58);
    PA_3DCreateSpriteFromTex(Quit,gfx[Quit],64,32,0,227,171);
    PA_3DCreateSpriteFromTex(Save,gfx[Save],64,32,0,227,136);
    PA_3DCreateSpriteFromTex(Score,gfx[Score],64,32,0,227,101);
    PA_3DProcess ();
}

void drawReplayGameInterface(void)
{
    PA_3DCreateSpriteFromTex(Up,gfx[Up],16,16,0,223,24);
    PA_3DCreateSpriteFromTex(Down,gfx[Down],16,16,0,223,60);
    PA_3DCreateSpriteFromTex(Right,gfx[Right],16,16,0,241,42);
    PA_3DCreateSpriteFromTex(Left,gfx[Left],16,16,0,205,42);
    PA_3DCreateSpriteFromTex(Quit,gfx[Quit],64,32,0,227,171);
    PA_3DProcess ();
}

void drawTsumegoInterface(void)
{
    PA_3DCreateSpriteFromTex(Left,gfx[Left],16,16,0,218,58);
    PA_3DCreateSpriteFromTex(Quit,gfx[Quit],64,32,0,227,171);
    PA_3DCreateSpriteFromTex(List,gfx[List],64,32,0,227,136);
    PA_3DProcess ();
}

void drawOkWrong(int ok)
{
    PA_3DCreateSpriteFromTex(10,
                            gfx[ok?Ok:Wrong],
                                     64, 64,
                                     0,
                                     90,90);
    PA_3DProcess();
}

void eraseInterface(void)
{
    PA_3DDeleteSprite(Left);
    PA_3DDeleteSprite(Right);
    PA_3DDeleteSprite(Up);
    PA_3DDeleteSprite(Down);

    PA_3DDeleteSprite(Quit);
    PA_3DDeleteSprite(List);
    PA_3DDeleteSprite(Save);
    PA_3DDeleteSprite(Score);

    PA_3DProcess ();
}

void activePad(SGFTree * tree)
{
    PA_3DSetSpriteVisible(Left, tree->lastnode->parent?1:0);
    PA_3DSetSpriteVisible(Right, tree->lastnode->child?1:0);
    PA_3DSetSpriteVisible(Down, tree->lastnode->next?1:0);
    if (tree->lastnode->parent)
        PA_3DSetSpriteVisible(Up, tree->lastnode->parent->child != tree->lastnode? 1:0);
    else
        PA_3DSetSpriteVisible(Up, 0);
}
