/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is TsumegoDS, a Go program for DS.                           *
 *                                                                   *
 * https://sourceforge.net/projects/tsumegods for more informations  *
 *                                                                   *
 * Copyright 2011 by Florian DUTHEIL, (Tryss from KGS                *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation - version 3 or          *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License in file COPYING for more details.      *
 *                                                                   *
 * You should have received a copy of the GNU General Public         *
 * License along with this program; if not, write to the Free        *
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA 02111, USA.                                            *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include "sgftree.h"
#include "go.h"
#include "drawing.h"
#include <stdlib.h>
#include <PA9.h>

void initGoban(Goban * goban, Infos * infos)
{
    int i,j;
    goban->size = 19;
    goban->goban = malloc( goban->size*sizeof(Stone*) );
    goban->move = 0;
    goban->infos = infos;

    for (i=0; i<goban->size; i++)
    {
        goban->goban[i] = malloc( goban->size * sizeof(Stone));
        for (j=0; j<goban->size; j++)
        {
            goban->goban[i][j].color = EMPTY;
            goban->goban[i][j].parent = NULL;
            goban->goban[i][j].child = NULL;
            goban->goban[i][j].liberty = 0;
            goban->goban[i][j].x = i;
            goban->goban[i][j].y = j;
        }
    }

    return;
}

void resetGoban(Goban * goban)
{
    int i,j;
    goban->move = 0;
    for (i=0; i<goban->size; i++)
    {
        for(j=0; j<goban->size; j++)
        {
            goban->goban[i][j].color = EMPTY;
            goban->goban[i][j].parent = NULL;
            goban->goban[i][j].child = NULL;
            goban->goban[i][j].liberty = 0;
        }
    }
}

void rebuildGoban( Goban * goban , SGFTree * tree )
{
    SGFNode * way[300];
    int count = 0;
    SGFNode * currNode = tree->lastnode;
    resetGoban(goban);

    while(currNode)
    {
        way[count] = currNode;
        currNode = currNode->parent;
        count++;
    }

    if (tree->root)
    {

        while ( count > 0 )
        {
            SGFProperty * prop = way[count-1]->props;
            int i, j;

            while (prop)
            {
                switch (prop->name)
                {
                case SGFB:
                    goban->move++;
                case SGFAB:
                    i = get_moveX(prop, 19);
                    j = get_moveY(prop, 19);
                    playStone(goban, BLACK, i,j);
                    break;
                case SGFW:
                    goban->move++;
                case SGFAW:
                    i = get_moveX(prop, 19);
                    j = get_moveY(prop, 19);
                    playStone(goban, WHITE, i,j);
                    break;
                default:
                    break;
                }
                prop = prop->next;
            }

            count--;
        }
    }
}

Stone * lastParent(Stone * stone)
{
    while (stone->parent)
    {
        stone = stone->parent;
    }

    return stone;
}

Stone * lastChild(Stone * stone)
{
    while (stone->child)
    {
        stone = stone->child;
    }
    return stone;
}

void playStone(Goban * goban, int color, int i, int j)
{
    int modScore;
    addStone(goban, color, i,j);
    modScore = checkStone(goban, color, i, j);
}

void addStone(Goban * goban, int color, int posX, int posY)
{
    goban->goban[posX][posY].color = color;
    goban->goban[posX][posY].liberty = 0;
    drawStone(posX, posY, color);
    if (posX>0)
    {
        if ( goban->goban[posX-1][posY].color == color )
        {
            Stone * futurChild = lastParent(&(goban->goban[posX-1][posY]));
            goban->goban[posX][posY].child = futurChild;
            futurChild->parent = &(goban->goban[posX][posY]);
            goban->goban[posX][posY].liberty += futurChild->liberty - 1;
        }
        else if ( goban->goban[posX-1][posY].color == 1+ !(color-1) )
        {

            Stone * enemyStone = lastParent(&(goban->goban[posX-1][posY]));
            (enemyStone->liberty--);
        }
        else
        {
            (goban->goban[posX][posY].liberty++);
        }
    }
    if (posX<goban->size-1)
    {
        if ( goban->goban[posX+1][posY].color == color )
        {
            Stone * futurChild = lastParent(&(goban->goban[posX+1][posY]));

            if (futurChild != &(goban->goban[posX][posY]))
            {
                Stone * futurParent = lastChild(&(goban->goban[posX][posY]));
                futurParent->child = futurChild;
                futurChild->parent = futurParent;
                goban->goban[posX][posY].liberty += futurChild->liberty - 1;
            }
            else
            {
                (goban->goban[posX][posY].liberty)--;
            }
        }
        else if ( goban->goban[posX+1][posY].color == 1+ !(color-1) )
        {
            Stone * enemyStone = lastParent(&(goban->goban[posX+1][posY]));
            (enemyStone->liberty)--;
        }
        else
        {
            goban->goban[posX][posY].liberty++;
        }
    }
    if (posY>0)
    {
        if ( goban->goban[posX][posY-1].color == color )
        {
            Stone * futurChild = lastParent(&(goban->goban[posX][posY-1]));
            if (futurChild != &(goban->goban[posX][posY]))
            {
                Stone * futurParent = lastChild(&(goban->goban[posX][posY]));
                futurParent->child = futurChild;
                futurChild->parent = futurParent;
                goban->goban[posX][posY].liberty += futurChild->liberty - 1;
            }
            else
            {
                (goban->goban[posX][posY].liberty)--;
            }
        }
        else if ( goban->goban[posX][posY-1].color == 1+ !(color-1) )
        {
            Stone * enemyStone = lastParent(&(goban->goban[posX][posY-1]));
            (enemyStone->liberty)--;
        }
        else
        {
            (goban->goban[posX][posY].liberty)++;
        }
    }
    if (posY<goban->size-1)
    {
        if ( goban->goban[posX][posY+1].color == color )
        {
            Stone * futurChild = lastParent(&(goban->goban[posX][posY+1]));
            if (futurChild != &(goban->goban[posX][posY]))
            {
                Stone * futurParent = lastChild(&(goban->goban[posX][posY]));
                futurParent->child = futurChild;
                futurChild->parent = futurParent;
                goban->goban[posX][posY].liberty += futurChild->liberty - 1;
            }
            else
            {
                (goban->goban[posX][posY].liberty)--;
            }
        }
        else if ( goban->goban[posX][posY+1].color == 1+ !(color-1) )
        {
            Stone * enemyStone = lastParent(&(goban->goban[posX][posY+1]));
            (enemyStone->liberty)--;
        }
        else
        {
            (goban->goban[posX][posY].liberty)++;
        }
    }
}

int removeStone (Goban * goban, Stone * parent, int color)
{
    int count = 0;
    Stone * currentStone = lastChild(parent);

    while ( currentStone )
    {
        Stone * nextStone = currentStone->parent;
        if (currentStone->x > 0)
        {
            if ( goban->goban[currentStone->x-1][currentStone->y].color == color )
            {
                Stone * stone = lastParent(&(goban->goban[currentStone->x-1][currentStone->y]));
                stone->liberty ++ ;
            }
        }
        if (currentStone->x < goban->size-1)
        {
            if ( goban->goban[currentStone->x+1][currentStone->y].color == color )
            {
                Stone * stone = lastParent(&(goban->goban[currentStone->x+1][currentStone->y]));
                stone->liberty ++ ;
            }
        }
        if (currentStone->y > 0)
        {
            if ( goban->goban[currentStone->x][currentStone->y-1].color == color )
            {
                Stone * stone = lastParent(&(goban->goban[currentStone->x][currentStone->y-1]));
                stone->liberty ++ ;
            }
        }
        if (currentStone->x < goban->size-1)
        {
            if ( goban->goban[currentStone->x][currentStone->y+1].color == color )
            {
                Stone * stone = lastParent(&(goban->goban[currentStone->x][currentStone->y+1]));
                stone->liberty ++ ;
            }
        }
        eraseStone(currentStone->x, currentStone->y);
        currentStone->child = NULL;
        currentStone->parent = NULL;
        currentStone->color = EMPTY;
        currentStone->liberty = 0;
        count ++;
        currentStone = nextStone;
    }
    return count;
}

int checkStone(Goban * goban, int color, int posX, int posY)
{
    int count = 0;
    if (posX>0)
    {
        if (goban->goban[posX-1][posY].color == 1+ !(color-1) )
        {

            Stone * parent = lastParent(&(goban->goban[posX-1][posY]));

            if ( parent->liberty == 0 )
            {
                count += removeStone(goban, parent, color);
            }
        }
    }
    if (posX<goban->size-1)
    {
        if (goban->goban[posX+1][posY].color == 1+ !(color-1) )
        {

            Stone * parent = lastParent(&(goban->goban[posX+1][posY]));

            if ( parent->liberty == 0 )
            {
                count += removeStone(goban, parent, color);
            }
        }
    }
    if (posY>0)
    {
        if (goban->goban[posX][posY-1].color == 1+ !(color-1) )
        {

            Stone * parent = lastParent(&(goban->goban[posX][posY-1]));

            if ( parent->liberty == 0 )
            {
                count += removeStone(goban, parent, color);
            }
        }
    }
    if (posY<goban->size-1)
    {
        if (goban->goban[posX][posY+1].color == 1+ !(color-1) )
        {

            Stone * parent = lastParent(&(goban->goban[posX][posY+1]));

            if ( parent->liberty == 0 )
            {
                count += removeStone(goban, parent, color);
            }
        }
    }
    if (goban->goban[posX][posY].liberty == 0)
    {
        count -= removeStone(goban, &(goban->goban[posX][posY]), 1+!(color-1) );
    }
    return count;
}


void fillGoban(Goban * goban)
{
    int x,y;
    PA_ClearTextBg(1);
    for (x=0; x<19; x++)
    {
        for (y=0; y<19; y++)
        {
            Stone * stone = &(goban->goban[x][y]);
            if (stone->color == EMPTY )
            {
                stone->color = NOT_KNOWN;
                if (y>0)
                {
                    Stone * neighbour = &(goban->goban[x][y-1]);
                    switch(neighbour->color)
                    {
                    case BLACK :
                        stone->color = BLACK_T;
                        break;
                    case WHITE :
                        stone->color = WHITE_T;
                        break;
                    default :
                        stone->color = goban->goban[x][y-1].color;
                        stone->child = lastParent(neighbour);
                        stone->child->parent = stone;
                    }
                }
                if (x>0)
                {
                    Stone * neighbour = &(goban->goban[x-1][y]);
                    Stone * head;
                    Stone * tail;
                    switch(neighbour->color)
                    {
                    case BLACK_T :
                        head = lastParent(neighbour);
                        tail = lastChild(stone);
                        if (head != stone)
                        {
                            tail->child = head;
                            head->parent = tail;
                        }
                    case BLACK :
                        switch(stone->color)
                        {
                        case WHITE_T:
                        case DAME:
                            stone->color = DAME;
                            break;
                        case BLACK_T:
                        case NOT_KNOWN:
                            stone->color = BLACK_T;
                            break;
                        }
                        break;
                    case WHITE_T :
                        head = lastParent(neighbour);
                        tail = lastChild(stone);
                        if (head != stone)
                        {
                            tail->child = head;
                            head->parent = tail;
                        }
                    case WHITE :
                        switch(stone->color)
                        {
                        case BLACK_T:
                        case DAME:
                            stone->color = DAME;
                            break;
                        case WHITE_T:
                        case NOT_KNOWN:
                            stone->color = WHITE_T;
                            break;
                        }
                        break;
                    case DAME :
                        head = lastParent(neighbour);
                        tail = lastChild(stone);
                        if (head != stone)
                        {
                            tail->child = head;
                            head->parent = tail;
                        }
                        stone->color = DAME;
                        break;
                    case NOT_KNOWN :
                        head = lastParent(neighbour);
                        tail = lastChild(stone);
                        if (head != stone)
                        {
                            tail->child = head;
                            head->parent = tail;
                        }
                    }
                }
                if (x<18)
                {
                    Stone * neighbour = &(goban->goban[x+1][y]);
                    switch(neighbour->color)
                    {
                    case BLACK :
                        switch(stone->color)
                        {
                        case WHITE_T:
                        case DAME:
                            stone->color = DAME;
                            break;
                        case BLACK_T:
                        case NOT_KNOWN:
                            stone->color = BLACK_T;
                            break;
                        }
                        break;
                    case WHITE :
                        switch(stone->color)
                        {
                        case BLACK_T:
                        case DAME:
                            stone->color = DAME;
                            break;
                        case WHITE_T:
                        case NOT_KNOWN:
                            stone->color = WHITE_T;
                            break;
                        }
                        break;
                    }
                }
                if (y<18)
                {
                    Stone * neighbour = &(goban->goban[x][y+1]);
                    switch(neighbour->color)
                    {
                    case BLACK :
                        switch(stone->color)
                        {
                        case WHITE_T:
                        case DAME:
                            stone->color = DAME;
                            break;
                        case BLACK_T:
                        case NOT_KNOWN:
                            stone->color = BLACK_T;
                            break;
                        }
                        break;
                    case WHITE :
                        switch(stone->color)
                        {
                        case BLACK_T:
                        case DAME:
                            stone->color = DAME;
                            break;
                        case WHITE_T:
                        case NOT_KNOWN:
                            stone->color = WHITE_T;
                            break;
                        }
                        break;
                    }

                }
                Stone * propagate = stone->child;
                while(propagate)
                {
                    propagate->color = propagate->parent->color;
                    propagate = propagate->child;
                }
            }
        }
    }
}

void countScore(Goban * goban, int * blackScore, int * whiteScore)
{
    int x,y;

    *blackScore = 0;
    *whiteScore = 0;

    for(x=0; x<19; x++)
    {
        for(y=0; y<19; y++)
        {
            switch (goban->goban[x][y].color)
            {
            case BLACK_T:
                drawCountPoint(x,y,BLACK);
            case BLACK:
                (*blackScore)++;
                break;

            case WHITE_T:
                drawCountPoint(x,y,WHITE);
            case WHITE:
                (*whiteScore)++;
                break;
            }
        }
    }
    PA_3DProcess();
}
