/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is TsumegoDS, a Go program for DS.                           *
 *                                                                   *
 * https://sourceforge.net/projects/tsumegods for more informations  *
 *                                                                   *
 * Copyright 2011 by Florian DUTHEIL, (Tryss from KGS                *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation - version 3 or          *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License in file COPYING for more details.      *
 *                                                                   *
 * You should have received a copy of the GNU General Public         *
 * License along with this program; if not, write to the Free        *
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA 02111, USA.                                            *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#ifndef __GO_H__
#define __GO_H__

#include "sgftree.h"

#define BLACK_T 3
#define WHITE_T 4
#define NOT_KNOWN 5
#define DAME 6

typedef struct infos_t
{
    int handicap;
    int komi;
    int mainTime;
    int periods;
    int additionnalTime;

    char black[50];
    char white[50];
    char blackRank[7];
    char whiteRank[7];
} Infos;

typedef struct stone_t
{
    int color;
    int x, y;
    struct stone_t * parent;
    struct stone_t * child;
    int liberty;
} Stone;

typedef struct goban_t
{
    int size;
    Stone ** goban;
    int move;
    Infos * infos;
} Goban;

void initGoban(Goban * goban, Infos * infos);
void resetGoban(Goban * goban);
void rebuildGoban( Goban * goban , SGFTree * tree );
Stone * lastParent(Stone * stone);
Stone * lastChild(Stone * stone);
void addStone(Goban * goban, int color, int posX, int posY);
int removeStone (Goban * goban, Stone * parent, int color);
int checkStone(Goban * goban, int color, int posX, int posY);
void playStone(Goban * goban, int color, int i, int j);
void fillGoban(Goban * goban);
void countScore(Goban * goban, int * blackScore, int * whiteScore);

#endif
