/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is TsumegoDS, a Go program for DS.                           *
 *                                                                   *
 * https://sourceforge.net/projects/tsumegods for more informations  *
 *                                                                   *
 * Copyright 2011 by Florian DUTHEIL, (Tryss from KGS                *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation - version 3 or          *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License in file COPYING for more details.      *
 *                                                                   *
 * You should have received a copy of the GNU General Public         *
 * License along with this program; if not, write to the Free        *
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA 02111, USA.                                            *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include <PA9.h>
#include "all_gfx.h"
#include "sgftree.h"
#include "recordGame.h"
#include "replayGame.h"
#include "go.h"
#include "drawing.h"
#include "tsumego.h"
#include "twoPlayers.h"

void intro(void);
void menu(void);

void test(void)
{
    int i;
    for (i=1; i<11; i++)
    {
        PA_16cText( 0,
                    0,
                    10*i,
                    50,
                    10*i+10,
                    "COLOR",
                    i, //color (1-10)
                    2, // text size (0-4)
                    100);
    }
}

int main(void)
{
    /* Init system */
    PA_Init();
    PA_InitVBL();
    fatInitDefault();

    /* Init graphics */
    PA_SetBgPalCol(0, 2, PA_RGB(0, 0, 0)); // BLACK
    PA_SetBgPalCol(0, 1, PA_RGB(31, 31, 31)); // WHITE
    PA_SetBgPalCol(0, 3, PA_RGB(31, 26, 14)); // BG
    PA_SetBgPalCol(0, 4, PA_RGB(31, 0, 0)); // RED

    initGraphics();

    PA_Init16cBg(0, 1);

    /* Init Bg */

    /* Init text */
    PA_InitText(1,2);
    PA_InitText(0,2);
    PA_SetTextCol(0,0,0,0);
    PA_SetTextCol(1,0,0,0);

    PA_SetBgPrioSeq(0,1,0,2,3);

    /* Intro */
    intro();

    /* Menu */
    menu();
    return 0;
}

void intro(void)
{
    return;
}

void menu(void)
{
    /* Load background */
    PA_LoadBackground(0,3,&Menu_BG);
    PA_LoadBackground(1,3,&Title_BG);

    while(1)
    {
        /* Wait click */
        PA_WaitFor(Stylus.Newpress || Pad.Newpress.Anykey);

        /* Record Game */
        if (PA_StylusInZone(34,12,216,51))
        {
            SGFTree tree;
            Infos infos;

            /* deleting old BG */
            PA_DeleteBg (0,3);
            PA_DeleteBg (1,3);

            if ( recordGameSetup( &tree, &infos ) )
            {
                recordGame( &tree, &infos );
            }
            /* Load background */
            PA_LoadBackground(0,3,&Menu_BG);
            PA_LoadBackground(1,3,&Title_BG);

        }
        else if(PA_StylusInZone(34,64,216,103))
        {
            SGFTree tree;

            /* deleting old BG */
            PA_DeleteBg (0,3);
            PA_DeleteBg (1,3);

            if ( loadGame( &tree ) )
            {
                replayGame( &tree );
            }
            /* Load background */
            PA_LoadBackground(0,3,&Menu_BG);
            PA_LoadBackground(1,3,&Title_BG);
        }
        else if(PA_StylusInZone(34,120,216,161))
        {
            SGFTree tree;
            int offset = 0;
            PA_DeleteBg (0,3);
            PA_DeleteBg (1,3);

            if (loadCollection(&tree))
            {
                if ( (offset = loadProblem(&tree,offset)) )
                {
                    tsumego(&tree, offset-1);
                }
            }
            /* Load background */
            PA_LoadBackground(0,3,&Menu_BG);
            PA_LoadBackground(1,3,&Title_BG);
        }
        else if(Pad.Held.L && Pad.Held.R)
        {
            initTwoPlayers();
            PA_LoadBackground(0,3,&Menu_BG);
            PA_LoadBackground(1,3,&Title_BG);
        }
    }
}
