/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is TsumegoDS, a Go program for DS.                           *
 *                                                                   *
 * https://sourceforge.net/projects/tsumegods for more informations  *
 *                                                                   *
 * Copyright 2011 by Florian DUTHEIL, (Tryss from KGS                *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation - version 3 or          *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License in file COPYING for more details.      *
 *                                                                   *
 * You should have received a copy of the GNU General Public         *
 * License along with this program; if not, write to the Free        *
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA 02111, USA.                                            *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include "recordGame.h"

int recordGameSetup( SGFTree * tree, Infos * infos )
{
    PA_LoadBackground(1,3,&Background);
    PA_LoadBackground(0,3,&Background);

    initInfos( infos );

    while(1)
    {
        writeInfos(infos);

        PA_WaitFor(Stylus.Newpress);

        if (PA_StylusInZone(0,5*8,255,6*8))
        {
            enterName(infos->black , "Black :");
        }
        else if (PA_StylusInZone(0,6*8,255,7*8))
        {
            enterName(infos->blackRank, "Black rank :");
        }
        else if (PA_StylusInZone(0,8*8,255,9*8))
        {
            enterName(infos->white , "White :");
        }
        else if (PA_StylusInZone(0,10*8,255,11*8))
        {
            enterName(infos->whiteRank, "White rank :");
        }
        else if PA_StylusInZone(0,12*8,255,13*8)
        {
            infos->handicap = enterNumber( "Handicap :" );
        }
        else if (PA_StylusInZone(0,14*8,255,15*8))
        {
            infos->komi = 10*enterNumber( "Komi :" ) + 5;
        }
        else if (PA_StylusInZone(0,16*8,255,17*8))
        {
            infos->mainTime = enterNumber( "Main time (in s) :");
        }
        else if (PA_StylusInZone(0,18*8,255,19*8))
        {
            infos->periods = enterNumber( "Periods :");
        }
        else if (PA_StylusInZone(0,20*8,255,21*8))
        {
            infos->additionnalTime = enterNumber( "Byo-Yomi :");
        }
        else if (PA_StylusInZone(19*8,22*8,24*8,23*8))
        {
            infosToSgf( infos, tree );
            PA_ClearTextBg(0);
            PA_DeleteBg (0,3);
            PA_DeleteBg (1,3);
            return 1;
        }
        else if (PA_StylusInZone(4*8,22*8,8*8,23*8))
        {
            PA_ClearTextBg(0);
            PA_DeleteBg (0,3);
            PA_DeleteBg (1,3);
            return 0;
        }
    }
}

void recordGame( SGFTree * tree, Infos * infos )
{
    Clock clock;
    Goban goban;
    int color;

    /* Init graphics */
    PA_LoadBackground(1,3,&Background);
    PA_LoadBackground(0,3,&Background19x19);

    drawRecordGameInterface();

    /* Init */
    initClock(&clock, infos->mainTime, infos->periods, infos->additionnalTime);
    initGoban(&goban, infos);

    /* Start game */
    color = BLACK;
    startClock(&clock, color);

    while(1)
    {
        PA_WaitForVBL();
        while(!(Stylus.Newpress))
        {
            updateClock(&clock);
            PA_WaitForVBL();
        }

        if ( PA_StylusInZone(3,3,192,192) )
        {
            if ( playMove( (Stylus.X-3)/10, ( Stylus.Y-3)/10, color, &goban, tree ) )
            {
                flipClock(&clock);
                color = color == BLACK? WHITE : BLACK;
            }
        }
        else if ( PA_3DSpriteTouched(Left))
        {
            if (tree->lastnode != tree->root)
            {
                treeBack(tree);
                sgfFreeNode(tree->lastnode->child);
                tree->lastnode->child = NULL;
                eraseAllStone();
                rebuildGoban(&goban, tree);
                PA_3DProcess();
                color = color == BLACK? WHITE:BLACK;
            }
        }
        else if ( PA_3DSpriteTouched(Score))
        {
            int blackScore, whiteScore;

            pauseClock( &clock);

            /* Remove Dead Stones */
            PA_WaitForVBL();
            PA_ClearTextBg(1);
            PA_OutputText(1,0,10,"Remove dead stones and touch score");
            PA_WaitFor(Stylus.Newpress);
            while(!PA_3DSpriteTouched(Score))
            {
                if (PA_StylusInZone(3,3,192,192))
                {
                    Stone * stone = &goban.goban[(Stylus.X-3)/10][(Stylus.Y-3)/10];
                    if (stone->color != EMPTY)
                        removeStone(&goban, stone, stone->color);
                        PA_3DProcess();
                }
                PA_WaitForVBL();
                PA_WaitFor(Stylus.Newpress);
            }

            /* Count score */
            fillGoban(&goban);
            countScore(&goban, &blackScore, &whiteScore);

            /* Write Score */
            PA_OutputText(1,0,0,"B: %d\nW: %d",blackScore,whiteScore);
            PA_WaitForVBL();
            PA_WaitFor(Stylus.Newpress);

            /* Return to old state */
            eraseAllStone();
            rebuildGoban(&goban, tree);
            PA_3DProcess();
            unpauseClock( &clock);
        }
        else if ( PA_3DSpriteTouched(Save) )
        {
            pauseClock(&clock);
            eraseAllStone();
            PA_3DProcess();

            saveGameToSGF( tree->root );

            rebuildGoban(&goban, tree);
            PA_3DProcess();
            unpauseClock(&clock);
        }
        else if ( PA_3DSpriteTouched(Quit) )
        {
            PA_DeleteBg(0,3);
            PA_DeleteBg(1,3);
            PA_ClearTextBg(1);
            eraseAll();
            return;
        }
    }
}

void initInfos( Infos * infos )
{
    strcpy(infos->black, "Black");
    strcpy(infos->white, "White");
    strcpy(infos->blackRank, "30k");
    strcpy(infos->whiteRank, "30k");
    infos->komi = 65;
    infos->mainTime = 20*60;
    infos->periods = 5;
    infos->additionnalTime = 30;
    infos->handicap = 0;
}

void writeInfos( Infos * infos )
{
    PA_OutputText(0,10,2,"Setup");
    PA_OutputText(0,2,4, "Black     :  %s      ", infos->black);
    PA_OutputText(0,2,6, "Black rank:  %s      ", infos->blackRank);
    PA_OutputText(0,2,8, "White     :  %s      ", infos->white);
    PA_OutputText(0,2,10,"White rank:  %s      ", infos->whiteRank);
    PA_OutputText(0,2,12,"Handicap  :  %d      ", infos->handicap);
    PA_OutputText(0,2,14,"Komi      :  %d.%d   ", infos->komi/10, infos->komi%10);
    PA_OutputText(0,2,16,"Main Time :  %d:%d   ", infos->mainTime/60, infos->mainTime%60);
    PA_OutputText(0,2,18,"Periods   :  %d      ", infos->periods);
    PA_OutputText(0,2,20,"Byo-Yomi  :  %d:%d   ", infos->additionnalTime/60, infos->additionnalTime%60);
    PA_OutputText(0,4,22,"Quit");
    PA_OutputText(0,19,22,"Start");
    PA_WaitForVBL();
}

void infosToSgf( Infos * infos, SGFTree * tree )
{
    tree->root = sgfNewNode();
    tree->lastnode = tree->root;
    sgfAddProperty(tree->root, "PB", infos->black);
    sgfAddProperty(tree->root, "BR", infos->blackRank);
    sgfAddProperty(tree->root, "PW", infos->white);
    sgfAddProperty(tree->root, "WR", infos->whiteRank);
    sgfAddPropertyInt(tree->root, "SZ", 19);
    sgfAddPropertyFloat(tree->root, "KM", infos->komi/10.0);
    sgfAddPropertyInt(tree->root, "HA", infos->handicap);
}

void saveGameToSGF( SGFNode * node )
{
    char filename[100] = "";
    char file[100];
    FILE * dump;
    enterName(filename, "Filename :");

    sprintf(file, "sgf/%s.sgf", filename);
    writesgf(node, file );

    dump = fopen("null","w");
    fclose(dump);
}

int enterNumber( char * str)
{
    int a;
    char text[20]="";
    s32 nletter = 0;
    char letter = 0;

    PA_ClearTextBg (1);
    PA_InitKeyboard(1);
    PA_KeyboardIn(0, 95);
    PA_OutputText(1,5,10, str);

    while ((letter = PA_CheckKeyboard()) != '\n')
    {
        if (letter >= '0' && letter <= '9')
        {
            text[nletter] = letter;
            text[nletter+1] = 0;
            nletter++;
        }
        else if ((letter == PA_BACKSPACE)&&nletter)
        {
            nletter--;
            text[nletter] = '\0';
        }

        PA_OutputText(1, 8, 11,"%s ", text);
        PA_WaitForVBL();
    }

    sscanf(text,"%d",&a);
    PA_KeyboardOut();
    PA_ClearTextBg (1);
    return a;
}

void enterName(char * name, char * str)
{
    s32 nletter = strlen(name);
    char letter = 0;

    PA_ClearTextBg (1);
    PA_InitKeyboard(1);
    PA_KeyboardIn(0, 95);
    PA_OutputText(1, 5, 10, str);

    while ((letter = PA_CheckKeyboard()) != '\n')
    {
        if (letter > 31)
        {
            name[nletter] = letter;
            name[nletter+1] = 0;
            nletter++;
        }
        else if ((letter == PA_BACKSPACE)&&nletter)
        {
            nletter--;
            name[nletter] = '\0';
        }

        PA_OutputText(1, 8, 11,"%s ", name);
        PA_WaitForVBL();
    }

    PA_KeyboardOut();
    PA_ClearTextBg (1);
}



int playMove( int x, int y, int color, Goban * goban, SGFTree * tree )
{
    if (goban->goban[x][y].color != EMPTY)
        return 0;

    tree->lastnode = sgfAddPlay(tree->lastnode, color, x,y);
    drawStone(x, y, color);
    playStone( goban, color, x, y);
    PA_3DProcess();
    return 1;
}
