/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is TsumegoDS, a Go program for DS.                           *
 *                                                                   *
 * https://sourceforge.net/projects/tsumegods for more informations  *
 *                                                                   *
 * Copyright 2011 by Florian DUTHEIL, (Tryss from KGS                *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation - version 3 or          *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License in file COPYING for more details.      *
 *                                                                   *
 * You should have received a copy of the GNU General Public         *
 * License along with this program; if not, write to the Free        *
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA 02111, USA.                                            *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include "replayGame.h"

void writeComment(SGFTree * tree)
{
    char buffer[1000];
    SGFProperty * prop = tree->lastnode->props;
    PA_ClearTextBg(1);
    while (prop)
    {
        int i,j,line;
        switch (prop->name)
        {
        case SGFC:
        case SGFGC:
            i=j=line=0;
            while(prop->value[i+j] && line<18)
            {
                if (prop->value[i+j] == '\n')
                {
                    strncpy(buffer,prop->value+i,j);
                    buffer[j] = 0;
                    PA_OutputText(1,2,2+line,"%s",buffer);
                    line++;
                    i=i+j+1;
                    j=0;
                }
                if (j>28)
                {
                    while(prop->value[i+j] != ' ')
                    {
                        j--;
                    }
                    strncpy(buffer,prop->value+i,j);
                    buffer[j] = 0;
                    PA_OutputText(1,2,2+line,"%s",buffer);
                    line++;
                    i=i+j+1;
                    j=0;
                }
                j++;
            }
            PA_OutputText(1,2,2+line,"%s",prop->value+i);
            PA_OutputText(1,5,0,"Comment:             ");
            return;
        }
        prop = prop->next;
    }
}

void addMark(SGFTree * tree)
{
    char x,y;
    char mark[2] = "x";
    SGFProperty * prop = tree->lastnode->props;
    PA_16cErase (0);
    while (prop)
    {
        switch (prop->name)
        {
        case SGFLB:
            sscanf(prop->value,"%c%c:%c",&x,&y,mark);
            PA_16cText( 0,
                        10*((y-'a'))+3,
                        10*((x-'a'))+3,
                        10*((y-'a')+1)+3,
                        10*((x-'a')+1)+3,
                        mark,
                        2, //color (1-10)
                        2, // text size (0-4)
                        100);
            break;
        }
        prop = prop->next;
    }
}

void writeInformations(Goban * goban)
{
    PA_OutputText(1,21,23,"Move: %d  ",goban->move);
    PA_OutputText(1,2,21,"Black: %s, %s",goban->infos->black, goban->infos->blackRank);
    PA_OutputText(1,2,22,"White: %s, %s",goban->infos->white, goban->infos->whiteRank);
}

int countListGames( void )
{
    DIR_ITER* dir = NULL;
    char filename[1024];
    struct stat st;
    int i=0;

    PA_WaitForVBL();

    dir=diropen("/sgf/");

    PA_WaitForVBL();
    if (dir==NULL) // Le dossier n'a pas pu être lu
    {
        return 0;
    }
    while (dirnext(dir,filename,&st) == 0)
    {
        if (! (st.st_mode&S_IFDIR) )
        {
            i++;
            PA_WaitForVBL();
        }
    }
    dirclose(dir);
    return i;
}

void writeListGames( int offset )
{
    DIR_ITER* dir = NULL;
    struct stat st;
    char filename[1024];
    int i = 0;

    dir=diropen("/sgf/");

    if (dir==NULL)
    {
        return;
    }
    else
    {
        while (dirnext(dir,filename,&st) == 0 && i-offset<7)
        {
            if (! (st.st_mode&S_IFDIR) )
            {
                if (i>=offset)
                {
                    PA_OutputText(0,5,2*(i-offset)+6,filename);
                }
                i++;
            }
        }
        dirclose(dir);
    }
}

void loadGameNumber(int number, SGFTree * tree)
{
    DIR_ITER* dir = NULL;
    struct stat st;
    char filename[1024];
    char buffer[1024];
    int i = 0;

    dir=diropen("/sgf/");

    if (dir==NULL)
    {
        return;
    }
    else
    {
        while (dirnext(dir,filename,&st) == 0 )
        {
            if (! (st.st_mode&S_IFDIR) )
            {
                if (i==number)
                {
                    sprintf(buffer,"sgf/%s",filename);
                    sgftree_readfile(tree, buffer);
                }
                i++;
            }
        }
        dirclose(dir);
    }
}

int loadGame( SGFTree * tree )
{
    int i=0;
    int size;
    int offset = 0;

    PA_LoadBackground(0,3,&Background);
    PA_LoadBackground(1,3,&Background);

    size = countListGames();

    if (size == 0)
    {
        return 0;
    }

    PA_WaitForVBL();

    while (1)
    {
        PA_OutputText(0,10,1,"Files");
        PA_OutputText(0,27,2,"Up");
        PA_OutputText(0,26,22,"Down");
        writeListGames( offset );

        PA_WaitForVBL();
        PA_WaitFor(Stylus.Newpress || Pad.Newpress.Anykey);

        if (PA_StylusInZone(26*8,22*8,30*8,23*8) || Pad.Newpress.Down)
        {
            if (offset < size-7)
            {
                offset++;
            }
            PA_ClearTextBg (0);
        }
        else if (PA_StylusInZone(27*8,2*8,29*8,3*8) || Pad.Newpress.Up)
        {
            if (offset>0)
            {
                offset--;
            }
            PA_ClearTextBg (0);
        }
        else
        {
            for(i = 0; i<7 && i<size-offset; i++)
            {
                if (PA_StylusInZone(64,2*i*8 + 6*8,200,2*i*8+7*8))
                {
                    loadGameNumber(i+offset, tree);
                    PA_ClearTextBg (0);
                    return 1;
                }
            }
        }
    }
}


void infosFromSgf(Infos * infos, SGFTree * tree)
{
    strcpy(infos->black, "");
    strcpy(infos->white, "");
    strcpy(infos->blackRank, "");
    strcpy(infos->whiteRank, "");
    infos->komi = 0;
    infos->mainTime = 0;
    infos->periods = 0;
    infos->additionnalTime = 0;
    infos->handicap = 0;

    SGFProperty * prop = tree->root->props;
    while (prop)
    {
        switch (prop->name)
        {
        case SGFBR:
            strcpy(infos->blackRank, prop->value);
            break;
        case SGFWR:
            strcpy(infos->whiteRank, prop->value);
            break;
        case SGFPB:
            strcpy(infos->black, prop->value);
            break;
        case SGFPW:
            strcpy(infos->white, prop->value);
            break;
        }
        prop = prop->next;
    }
    sgfGetIntProperty(tree->root,"HA",&infos->handicap);
}

void replayGame( SGFTree * tree)
{
    Goban goban;
    Infos infos;

    infosFromSgf(&infos, tree);
    initGoban(&goban, &infos);

    /* Init graphics */
    PA_LoadBackground(1,3,&Background);
    PA_LoadBackground(0,3,&Background19x19);

    drawReplayGameInterface();

    tree->lastnode = tree->root;
    while (1)
    {
        eraseAllStone();
        rebuildGoban(&goban,tree);
        activePad(tree);
        PA_3DProcess();

        addMark(tree);
        writeComment(tree);
        writeInformations(&goban);

        PA_WaitForVBL();
        PA_WaitFor( Stylus.Newpress || Pad.Newpress.Anykey  );

        if (PA_3DSpriteTouched(Quit))
        {
            PA_DeleteBg(0,3);
            PA_DeleteBg(1,3);
            PA_ClearTextBg(1);
            eraseAll();
            PA_16cErase (0);

            return;
        }
        else if (PA_3DSpriteTouched(Right)  || Pad.Newpress.Right)
        {
            treeForward(tree);
        }
        else if (PA_3DSpriteTouched(Left) || Pad.Newpress.Left)
        {
            treeBack(tree);
        }
        else if (PA_3DSpriteTouched(Up) || Pad.Newpress.Up)
        {
            treeUp(tree);
        }
        else if (PA_3DSpriteTouched(Down) || Pad.Newpress.Down)
        {
            treeDown(tree);
        }
        else if (Pad.Newpress.X)
        {
            int i;
            for (i=0; i<10; i++)
            {
                treeForward(tree);
            }
        }
        else if (Pad.Newpress.Y)
        {
            int i;
            for (i=0; i<10; i++)
            {
                treeBack(tree);
            }
        }

    }
}
