/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is TsumegoDS, a Go program for DS.                           *
 *                                                                   *
 * https://sourceforge.net/projects/tsumegods for more informations  *
 *                                                                   *
 * Copyright 2011 by Florian DUTHEIL, (Tryss from KGS                *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation - version 3 or          *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License in file COPYING for more details.      *
 *                                                                   *
 * You should have received a copy of the GNU General Public         *
 * License along with this program; if not, write to the Free        *
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA 02111, USA.                                            *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include "tsumego.h"

int countListCollections( void )
{
    DIR_ITER* dir = NULL;
    char filename[1024];
    struct stat st;
    int i=0;

    PA_WaitForVBL();

    dir=diropen("/sgf/problems");

    PA_WaitForVBL();
    if (dir==NULL)
    {
        return 0;
    }
    while (dirnext(dir,filename,&st) == 0)
    {
        if (! (st.st_mode&S_IFDIR) )
        {
            i++;
            PA_WaitForVBL();
        }
    }
    dirclose(dir);
    return i;
}

int countListProblems( SGFTree * tree )
{
    int i=0;
    SGFNode * node = tree->root->child;
    while (node)
    {
        node = node->next;
        i++;
    }
    return i;
}

void writeListCollections( int offset )
{
    DIR_ITER* dir = NULL;
    struct stat st;
    char filename[1024];
    char buffer[1024];
    int i = 0;

    dir=diropen("/sgf/problems");

    if (dir==NULL)
    {
        return;
    }
    else
    {
        while (dirnext(dir,filename,&st) == 0 && i-offset<7)
        {
            if (! (st.st_mode&S_IFDIR) )
            {
                if (i>=offset)
                {
                    sscanf(filename,"%s.sgf",buffer);
                    PA_OutputText(0,5,2*(i-offset)+6,filename);
                }
                i++;
            }
        }
        dirclose(dir);
    }
}

void writeListProblems(int offset, int size)
{
    int i;
    for (i=offset; i-offset <7 && i<size; i++)
    {
        PA_OutputText(0,5,2*(i-offset)+6,"Problem %d", i+1);
    }
}

void loadCollectionNumber(int number, SGFTree * tree)
{
    DIR_ITER* dir = NULL;
    struct stat st;
    char filename[1024];
    char buffer[1024];
    int i = 0;

    dir=diropen("/sgf/problems");

    if (dir==NULL)
    {
        return;
    }
    else
    {
        while (dirnext(dir,filename,&st) == 0 )
        {
            if (! (st.st_mode&S_IFDIR) )
            {
                if (i==number)
                {
                    sprintf(buffer,"sgf/problems/%s",filename);
                    sgftree_readfile(tree, buffer);
                }
                i++;
            }
        }
        dirclose(dir);
    }
}

int loadProblemNumber(int number, SGFTree * tree)
{
    int i=0;
    tree->lastnode = tree->root->child;
    while(i!= number)
    {
        tree->lastnode=tree->lastnode->next;
        i++;
    }
    return 0;
}

int loadCollection( SGFTree * tree)
{
    int i=0;
    int size;
    int offset = 0;

    PA_LoadBackground(0,3,&Background);
    PA_LoadBackground(1,3,&Background);

    size = countListCollections();

    if (size == 0)
    {
        return 0;
    }

    PA_WaitForVBL();

    while (1)
    {
        PA_OutputText(0,2,1,"Collections of problems");
        PA_OutputText(0,27,2,"Up");
        PA_OutputText(0,26,22,"Down");
        writeListCollections( offset );

        PA_WaitForVBL();
        PA_WaitFor(Stylus.Newpress || Pad.Newpress.Anykey);

        if (PA_StylusInZone(26*8,22*8,30*8,23*8) || Pad.Newpress.Down)
        {
            if (offset < size-7)
            {
                offset++;
            }
            PA_ClearTextBg (0);
        }
        else if (PA_StylusInZone(27*8,2*8,29*8,3*8) || Pad.Newpress.Up)
        {
            if (offset>0)
            {
                offset--;
            }
            PA_ClearTextBg (0);
        }
        else
        {
            for(i = 0; i<7 && i<size-offset; i++)
            {
                if (PA_StylusInZone(64,2*i*8 + 6*8,200,2*i*8+7*8))
                {
                    loadCollectionNumber(i+offset, tree);
                    PA_ClearTextBg (0);
                    return 1;
                }
            }
        }
    }
}

int loadProblem( SGFTree * tree, int offset)
{
    int i=0;
    int size;

    size = countListProblems( tree );

    if (size == 0)
    {
        return 0;
    }

    PA_WaitForVBL();

    while (1)
    {
        PA_OutputText(0,7,1,"Problems");
        PA_OutputText(0,27,2,"Up");
        PA_OutputText(0,26,22,"Down");
        writeListProblems( offset, size );

        PA_WaitForVBL();
        PA_WaitFor(Stylus.Newpress || Pad.Newpress.Anykey);

        if (PA_StylusInZone(26*8,22*8,30*8,23*8) || Pad.Newpress.Down)
        {
            if (offset < size-7)
            {
                offset++;
            }
            PA_ClearTextBg (0);
        }
        else if (PA_StylusInZone(27*8,2*8,29*8,3*8) || Pad.Newpress.Up)
        {
            if (offset>0)
            {
                offset--;
            }
            PA_ClearTextBg (0);
        }
        else
        {
            for(i = 0; i<7 && i<size-offset; i++)
            {
                if (PA_StylusInZone(64,2*i*8 + 6*8,200,2*i*8+7*8))
                {
                    loadProblemNumber(i+offset, tree);
                    PA_ClearTextBg (0);
                    return offset+1;
                }
            }
        }
    }
}

void tsumego(SGFTree * tree, int offset)
{
    Goban goban;
    Infos infos;

    infosFromSgf(&infos, tree);
    initGoban(&goban, &infos);

    /* Init graphics */
    PA_LoadBackground(1,3,&Background);
    PA_LoadBackground(0,3,&Background19x19);

    drawTsumegoInterface();

    while (1)
    {
        eraseAllStone();
        rebuildGoban(&goban,tree);
        PA_3DProcess();

        addMark(tree);
        writeComment(tree);
        writeInformations(&goban);

        PA_WaitForVBL();
        PA_WaitFor( Stylus.Newpress || Pad.Newpress.Anykey  );

        if (PA_3DSpriteTouched(Quit))
        {
            PA_DeleteBg(0,3);
            PA_DeleteBg(1,3);
            PA_ClearTextBg(1);
            eraseAll();
            PA_16cErase (0);

            return;
        }
        else if (PA_3DSpriteTouched(List))
        {
            PA_DeleteBg(0,3);
            PA_DeleteBg(1,3);
            PA_ClearTextBg(1);
            eraseAll();
            PA_16cErase (0);
            PA_LoadBackground(0,3,&Background);
            PA_LoadBackground(1,3,&Background);

            loadProblem( tree, offset);

            PA_DeleteBg(0,3);
            PA_DeleteBg(1,3);
            PA_LoadBackground(1,3,&Background);
            PA_LoadBackground(0,3,&Background19x19);
            drawTsumegoInterface();
        }
        else if (PA_3DSpriteTouched(Left) || Pad.Newpress.Left)
        {
            if (tree->lastnode->parent != tree->root)
            {
                treeBack(tree);
            }
        }
        else if (PA_StylusInZone(3,3,192,192))
        {
            tryToMove( tree, &goban, (Stylus.X-3)/10, (Stylus.Y-3)/10 );
            testEndOfProblem( tree );
        }

    }
}

void tryToMove(SGFTree * tree, Goban * goban, int x, int y)
{
    int i, j, turn=EMPTY;
    SGFNode * currNode = tree->lastnode->child;
    while (currNode)
    {
        SGFProperty * prop = currNode->props;

        while (prop)
        {
            if (prop->name == SGFB)
            {
                turn = BLACK;
            }
            if (prop->name == SGFW)
            {
                turn = WHITE;
            }
            switch (prop->name)
            {
            case SGFB:
            case SGFW:
                i = get_moveX(prop, 19);
                j = get_moveY(prop, 19);
                if (i==x && j==y)
                {
                    if (currNode->child)
                    {
                        tree->lastnode = currNode->child;
                    }
                    else
                    {
                        tree->lastnode = currNode;
                    }
                    drawStone(x, y, turn);
                    return;
                }
                break;
            default:
                break;
            }
            prop = prop->next;
        }
        currNode = currNode->next;
    }
    if (goban->goban[x][y].color == EMPTY)
    {
        tree->lastnode = sgfAddPlay(tree->lastnode, turn, x,y);
        drawStone(x, y, turn);
    }
}

void testEndOfProblem( SGFTree * tree )
{
    SGFProperty * prop = tree->lastnode->props;

    if (tree->lastnode->child)
    {
        return;
    }

    while (prop)
    {
        switch (prop->name)
        {
        case SGFC:
            drawOkWrong(strcmp(prop->value, "CORRECT") == 0);
            PA_WaitForVBL();
            PA_WaitFor(Stylus.Newpress || Pad.Newpress.Anykey);
            PA_3DDeleteSprite (10);
            PA_3DProcess();
            return;
        default:
            break;
        }
        prop = prop->next;
    }

    drawOkWrong(0);
    PA_WaitForVBL();
    PA_WaitFor(Stylus.Newpress|| Pad.Newpress.Anykey);
    PA_3DDeleteSprite (10);
    PA_3DProcess();
    return;
}
