/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is TsumegoDS, a Go program for DS.                           *
 *                                                                   *
 * https://sourceforge.net/projects/tsumegods for more informations  *
 *                                                                   *
 * Copyright 2011 by Florian DUTHEIL, (Tryss from KGS                *
 *                                                                   *
 * This program is free software; you can redistribute it and/or     *
 * modify it under the terms of the GNU General Public License as    *
 * published by the Free Software Foundation - version 3 or          *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program is distributed in the hope that it will be useful,   *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
 * GNU General Public License in file COPYING for more details.      *
 *                                                                   *
 * You should have received a copy of the GNU General Public         *
 * License along with this program; if not, write to the Free        *
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA 02111, USA.                                            *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#include "twoPlayers.h"

int initWifi(void)
{
    return PA_InitWifi() && PA_ConnectWifiWFC();
}

int createServer(void)
{

    struct sockaddr_in serverAddress; // information du serveur
    struct sockaddr_in ClientAddress; //information du client

    serverAddress.sin_family = AF_INET;
    //pour la famille nous utilisons AF_INET

    serverAddress.sin_port = htons(23);
    //pour le port nous utilisons la fonction htons() avec en paramètre le numéro du port

    serverAddress.sin_addr.s_addr = 0;
    //pour l'adresse ip nous mettons 0 car une ip a déjà été attribué a la ds.

    int sock = 0;
    sock = socket(AF_INET, SOCK_STREAM, 0);

    bind(   sock, // le descripteur de socket
            (struct sockaddr *)&serverAddress, // les infos du serveur
            sizeof(serverAddress)); // la taille de la structure des info du serveur

    listen( sock, // le descripteur de socket
            1); // le nombre de connection autorisée

    int csock = 0;
    fd_set readfs;
    do
    {
        FD_ZERO(&readfs);
        FD_SET(sock, &readfs);

        /* Si une erreur est survenue au niveau du select */
        if(select(sock + 1, &readfs, NULL, NULL, NULL) < 0)
        {
            return 0;
        }

        if(FD_ISSET(sock, &readfs))
        {
            csock = accept( sock, //ancien descripteur
                            (struct sockaddr *)&ClientAddress,// recuperation des info client
                            (int*)sizeof(struct sockaddr_in));
        }
        PA_WaitForVBL();
    }
    while ( !csock && !Pad.Newpress.Start);
    return csock;
}

int createClient(char * ip)
{
    int sock;
    PA_InitSocket(&sock,ip,23,PA_NORMAL_TCP);
    return sock;
}

int messages( int sock )
{
    fd_set readfs;
    char buffer[256];
    FD_ZERO(&readfs);
    FD_SET(sock, &readfs);

    /* Si une erreur est survenue au niveau du select */
    if(select(sock + 1, &readfs, NULL, NULL, NULL) < 0)
    {
        return NOTHING;
    }

    if(FD_ISSET(sock, &readfs))
    {
        recv(sock,buffer,256,0);

        if(!strcmp(buffer,"INFOS"))
        {
            return INFOS;
        }
        else if (!strcmp(buffer,"ACCEPT"))
        {
            return ACCEPT;
        }
        else if (!strcmp(buffer,"REFUSE"))
        {
            return REFUSE;
        }
        else if (!strcmp(buffer,"MOVE"))
        {
            return MOVE;
        }
        else if (!strcmp(buffer,"QUIT"))
        {
            return QUIT;
        }
    }
    return NOTHING;
}



void init2PInfos( Infos * infos )
{
    strcpy(infos->black, "SERVER");
    strcpy(infos->white, "CLIENT");
    infos->komi = 65;
    infos->mainTime = 20*60;
    infos->periods = 5;
    infos->additionnalTime = 30;
    infos->handicap = 0;
}

void write2PInfos( Infos * infos )
{
    PA_OutputText(0,10,2,"Setup");
    PA_OutputText(0,2,4, "Color     :  %s      ", strcmp(infos->black,"SERVER")?"White":"Black" );
    PA_OutputText(0,2,6,"Handicap  :  %d      ", infos->handicap);
    PA_OutputText(0,2,8,"Komi      :  %d.%d   ", infos->komi/10, infos->komi%10);
    PA_OutputText(0,2,10,"Main Time :  %d:%d   ", infos->mainTime/60, infos->mainTime%60);
    PA_OutputText(0,2,12,"Periods   :  %d      ", infos->periods);
    PA_OutputText(0,2,14,"Byo-Yomi  :  %d:%d   ", infos->additionnalTime/60, infos->additionnalTime%60);
    PA_OutputText(0,4,22,"Quit");
    PA_OutputText(0,19,22,"Start");
    PA_WaitForVBL();
}

int record2PGameSetup( Infos * infos )
{
    PA_LoadBackground(1,3,&Background);
    PA_LoadBackground(0,3,&Background);

    initInfos( infos );

    while(1)
    {
        write2PInfos(infos);

        PA_WaitFor(Stylus.Newpress);

        if (PA_StylusInZone(0,5*8,255,6*8))
        {
            char buffer[50];
            enterName(buffer , "Your color :");
            if(strcmp(buffer,"black"))
            {
                strcpy(infos->white,"SERVER");
                strcpy(infos->black,"CLIENT");
            }
            else
            {
                strcpy(infos->white,"CLIENT");
                strcpy(infos->black,"SERVER");
            }
        }
        else if PA_StylusInZone(0,7*8,255,8*8)
        {
            infos->handicap = enterNumber( "Handicap :" );
        }
        else if (PA_StylusInZone(0,9*8,255,10*8))
        {
            infos->komi = 10*enterNumber( "Komi :" ) + 5;
        }
        else if (PA_StylusInZone(0,11*8,255,12*8))
        {
            infos->mainTime = enterNumber( "Main time (in s) :");
        }
        else if (PA_StylusInZone(0,13*8,255,14*8))
        {
            infos->periods = enterNumber( "Periods :");
        }
        else if (PA_StylusInZone(0,15*8,255,16*8))
        {
            infos->additionnalTime = enterNumber( "Byo-Yomi :");
        }
        else if (PA_StylusInZone(19*8,22*8,24*8,23*8))
        {
            PA_ClearTextBg(0);
            return 1;
        }
        else if (PA_StylusInZone(4*8,22*8,8*8,23*8))
        {
            PA_ClearTextBg(0);
            return 0;
        }
    }
}

int getInfos(int sock, Infos * infos)
{
    char buffer[256];
    char c;

    PA_ClearTextBg(0);
    recv(sock,buffer,256,0);

    sscanf(buffer,"CO:%c HA:%d KO:%d MT:%d PE:%d BY:%d",
           &c,
           &infos->handicap,
           &infos->komi,
           &infos->mainTime,
           &infos->periods,
           &infos->additionnalTime);
    strcpy(infos->black,c=='B'?"SERVER":"CLIENT");
    strcpy(infos->white,c=='B'?"CLIENT":"SERVER");

    write2PInfos( infos );

    do
    {
        PA_WaitForVBL();
        PA_WaitFor(Stylus.Newpress);
        if (PA_StylusInZone(4*8,22*8,8*8,23*8))
        {
            strcpy(buffer,"REFUSE");
            send(sock,buffer,256,0);
            return 0;
        }
        else if (PA_StylusInZone(19*8,22*8,24*8,23*8))
        {
            strcpy(buffer,"ACCEPT");
            send(sock,buffer,256,0);
            return 1;
        }
    }
    while(1);
}

int sendInfos(int sock, Infos * infos)
{
    char buffer[256];
    int message;

    int ok = record2PGameSetup( infos );

    if(ok==0)
    {
        return -1;
    }

    /* Send infos */
    strcpy(buffer,"INFOS");
    send(sock,buffer,256,0);

    sprintf(buffer,"CO:%c HA:%d KO:%d MT:%d PE:%d BY:%d",
           strcmp(infos->black,"SERVER")?'W':'B',
           infos->handicap,
           infos->komi,
           infos->mainTime,
           infos->periods,
           infos->additionnalTime);
    send(sock,buffer,256,0);

    /* Wait confirmation */
    PA_OutputText(0,0,0,"Waiting confirmation");
    do
    {
        PA_WaitForVBL();
        message = messages(sock);
        if (Pad.Newpress.Start)
        {
            return -1;
        }
    }
    while(message != ACCEPT && message != REFUSE );

    return message == ACCEPT?1:0;
}

void getMove(int sock, int *x, int *y)
{
    char buffer[256];
    recv(sock,buffer,256,0);
    *x = buffer[0]-'a';
    *y = buffer[1]-'a';
}

void sendMove(int sock, int x, int y)
{
    char buffer[256];
    strcpy(buffer,"MOVE");
    send(sock,buffer, 256,0);
    buffer[0]=x+'a';
    buffer[1]=y+'a';
    buffer[2]=0;
    send(sock,buffer, 256,0);
}

int quit(int sock)
{
    char buffer[256];
    strcpy(buffer,"QUIT");
    send(sock,buffer,256,0);
    return 0;
}

int protocol(int sock, int isServer)
{
    int message;
    int ok;
    Infos infos;

    do
    {
        if (isServer)
        {
            ok = sendInfos(sock,&infos);
            if (ok < 0)
            {
                quit(sock);
                return 0;
            }
            if (!ok)
            {
                isServer = 0;
            }
        }
        else
        {
            PA_ClearTextBg(0);
            PA_OutputText(0,0,0,"Wait Infos");
            do
            {
                message = messages(sock);
                PA_WaitForVBL();
                if (Pad.Newpress.Start || message == QUIT)
                {
                    quit(sock);
                    return 0;
                }
            }
            while(message != INFOS);
            ok = getInfos(sock,&infos);
            if (!ok)
            {
                isServer = 1;
            }
        }
    }
    while(!ok);

    PA_ClearTextBg(0);
    twoPlayerGame(sock,isServer,&infos);
    return 1;
}

void twoPlayerGame( int sock, int isServer, Infos * infos)
{
    SGFTree tree;
    Clock clock;
    Goban goban;
    int color;
    int player;

    if (strcmp(infos->black,"SERVER")==0)
    {
        player = isServer?BLACK:WHITE;
    }
    else
    {
        player = isServer?WHITE:BLACK;
    }

    /* Init graphics */
    PA_LoadBackground(1,3,&Background);
    PA_LoadBackground(0,3,&Background19x19);

    drawRecordGameInterface();

    /* Init */
    initClock(&clock, infos->mainTime, infos->periods, infos->additionnalTime);
    initGoban(&goban, infos);

    /* Start game */
    color = BLACK;
    startClock(&clock, color);

    while(1)
    {
        PA_WaitForVBL();

        if (color == player)
        {
            while(!(Stylus.Newpress))
            {
                updateClock(&clock);
                PA_WaitForVBL();
            }

            if ( PA_StylusInZone(3,3,192,192) )
            {
                if ( playMove( (Stylus.X-3)/10, ( Stylus.Y-3)/10, color, &goban, &tree ) )
                {

                    sendMove(sock, (Stylus.X-3)/10, ( Stylus.Y-3)/10);
                    flipClock(&clock);
                    color = color == BLACK? WHITE : BLACK;
                }
            }
        }
        else
        {
            int message = messages(sock);
            if (message==MOVE)
            {
                int x, y;
                getMove(sock, &x, &y);
                playMove(x,y,color, &goban, &tree);
                flipClock(&clock);
                color = color == BLACK? WHITE : BLACK;
            }
        }
        if ( PA_3DSpriteTouched(Quit) )
        {
            PA_DeleteBg(0,3);
            PA_DeleteBg(1,3);
            PA_ClearTextBg(1);
            eraseAll();
            return;
        }
    }
}

int initTwoPlayers(void)
{
    PA_LoadBackground(0,3,&Background);
    PA_LoadBackground(1,3,&Background);

    PA_OutputText(0,0,0,"Initializing WiFi...");
    while(!initWifi())
    {
        PA_OutputText(0,0,2,"Error in WiFi initialization\nRetry...");
    }

    int sock;
    int isServer;

    PA_ClearTextBg(0);
    PA_OutputText(0,0,0,"A pour mode server");
    PA_OutputText(0,0,1,"B pour mode client");
    PA_WaitForVBL();
    PA_WaitFor(Pad.Newpress.A || Pad.Newpress.B);
    if (Pad.Newpress.A)
    {
        sock = createServer();
        isServer = 1;
    }
    else
    {
        char ip[20];
        enterName(ip, "IP Adress to connect :");
        sock = createClient(ip);
        isServer = 0;
    }
    PA_ClearTextBg(0);
    protocol(sock, isServer);
    PA_ClearTextBg(0);
    Wifi_DisconnectAP();
    Wifi_DisableWifi();
    return 0;
}
